@@ -16,15 +16,15 @@
 
 import abc
 import collections
+from collections import abc as collections_abc
 import copy
+import functools
 import logging
 import warnings
 
 import oslo_messaging as messaging
-from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import versionutils as vutils
-import six
 
 from oslo_versionedobjects._i18n import _
 from oslo_versionedobjects import exception
@@ -34,7 +34,7 @@ from oslo_versionedobjects import fields as obj_fields
 LOG = logging.getLogger('object')
 
 
-class _NotSpecifiedSentinel(object):
+class _NotSpecifiedSentinel:
     pass
 
 
@@ -83,7 +83,7 @@ def _make_class_properties(cls):
                 return setattr(self, attrname, field_value)
             except Exception:
                 with excutils.save_and_reraise_exception():
-                    attr = "%s.%s" % (self.obj_name(), name)
+                    attr = "{}.{}".format(self.obj_name(), name)
                     LOG.exception('Error setting %(attr)s',
                                   {'attr': attr})
 
@@ -96,7 +96,7 @@ def _make_class_properties(cls):
         setattr(cls, name, property(getter, setter, deleter))
 
 
-class VersionedObjectRegistry(object):
+class VersionedObjectRegistry:
     _registry = None
 
     def __new__(cls, *args, **kwargs):
@@ -167,7 +167,7 @@ class VersionedObjectRegistry(object):
 # requested action and the result will be returned here.
 def remotable_classmethod(fn):
     """Decorator for remotable classmethods."""
-    @six.wraps(fn)
+    @functools.wraps(fn)
     def wrapper(cls, context, *args, **kwargs):
         if cls.indirection_api:
             version_manifest = obj_tree_get_versions(cls.obj_name())
@@ -199,7 +199,7 @@ def remotable_classmethod(fn):
 # "orphaned" and remotable methods cannot be called.
 def remotable(fn):
     """Decorator for remotable object methods."""
-    @six.wraps(fn)
+    @functools.wraps(fn)
     def wrapper(self, *args, **kwargs):
         ctxt = self._context
         if ctxt is None:
@@ -230,7 +230,7 @@ def remotable(fn):
     return wrapper
 
 
-class VersionedObject(object):
+class VersionedObject:
     """Base class and object factory.
 
     This forms the base of all objects that can be remoted or instantiated
@@ -307,15 +307,13 @@ class VersionedObject(object):
             setattr(self, key, kwargs[key])
 
     def __repr__(self):
-        repr_str = '%s(%s)' % (
+        repr_str = '{}({})'.format(
             self.obj_name(),
-            ','.join(['%s=%s' % (name,
-                                 (self.obj_attr_is_set(name) and
-                                  field.stringify(getattr(self, name)) or
-                                  '<?>'))
+            ','.join(['{}={}'.format(name,
+                                     (self.obj_attr_is_set(name) and
+                                      field.stringify(getattr(self, name)) or
+                                      '<?>'))
                       for name, field in sorted(self.fields.items())]))
-        if six.PY2:
-            repr_str = encodeutils.safe_encode(repr_str, incoming='utf-8')
         return repr_str
 
     def __contains__(self, name):
@@ -345,7 +343,7 @@ class VersionedObject(object):
 
     @classmethod
     def _obj_primitive_key(cls, field):
-        return '%s.%s' % (cls.OBJ_SERIAL_NAMESPACE, field)
+        return '{}.{}'.format(cls.OBJ_SERIAL_NAMESPACE, field)
 
     @classmethod
     def _obj_primitive_field(cls, primitive, field,
@@ -397,7 +395,7 @@ class VersionedObject(object):
             if name in objdata:
                 setattr(self, name, field.from_primitive(self, name,
                                                          objdata[name]))
-        self._changed_fields = set([x for x in changes if x in self.fields])
+        self._changed_fields = {x for x in changes if x in self.fields}
         return self
 
     @classmethod
@@ -410,7 +408,7 @@ class VersionedObject(object):
             # NOTE(danms): We don't do anything with this now, but it's
             # there for "the future"
             raise exception.UnsupportedObjectError(
-                objtype='%s.%s' % (objns, objname))
+                objtype='{}.{}'.format(objns, objname))
         objclass = cls.obj_class_from_name(objname, objver)
         return objclass._obj_from_primitive(context, objver, primitive)
 
@@ -612,8 +610,8 @@ class VersionedObject(object):
 
     def obj_what_changed(self):
         """Returns a set of fields that have been modified."""
-        changes = set([field for field in self._changed_fields
-                       if field in self.fields])
+        changes = {field for field in self._changed_fields
+                   if field in self.fields}
         for field in self.fields:
             if (self.obj_attr_is_set(field) and
                     isinstance(getattr(self, field), VersionedObject) and
@@ -693,7 +691,7 @@ class VersionedObject(object):
         return self._context
 
 
-class ComparableVersionedObject(object):
+class ComparableVersionedObject:
     """Mix-in to provide comparison methods
 
     When objects are to be compared with each other (in tests for example),
@@ -707,7 +705,7 @@ class ComparableVersionedObject(object):
         return NotImplemented
 
     def __hash__(self):
-        return super(ComparableVersionedObject, self).__hash__()
+        return super().__hash__()
 
     def __ne__(self, obj):
         if hasattr(obj, 'obj_to_primitive'):
@@ -715,7 +713,7 @@ class ComparableVersionedObject(object):
         return NotImplemented
 
 
-class TimestampedObject(object):
+class TimestampedObject:
     """Mixin class for db backed objects with timestamp fields.
 
     Sqlalchemy models that inherit from the oslo_db TimestampMixin will include
@@ -727,7 +725,7 @@ class TimestampedObject(object):
     }
 
 
-class VersionedObjectDictCompat(object):
+class VersionedObjectDictCompat:
     """Mix-in to provide dictionary key access compatibility
 
     If an object needs to support attribute access using
@@ -743,33 +741,16 @@ class VersionedObjectDictCompat(object):
                     name in self.obj_extra_fields):
                 yield name
 
-    iterkeys = __iter__
+    keys = __iter__
 
-    def itervalues(self):
+    def values(self):
         for name in self:
             yield getattr(self, name)
 
-    def iteritems(self):
+    def items(self):
         for name in self:
             yield name, getattr(self, name)
 
-    if six.PY3:
-        # NOTE(haypo): Python 3 dictionaries don't have iterkeys(),
-        # itervalues() or iteritems() methods. These methods are provided to
-        # ease the transition from Python 2 to Python 3.
-        keys = iterkeys
-        values = itervalues
-        items = iteritems
-    else:
-        def keys(self):
-            return list(self.iterkeys())
-
-        def values(self):
-            return list(self.itervalues())
-
-        def items(self):
-            return list(self.iteritems())
-
     def __getitem__(self, name):
         return getattr(self, name)
 
@@ -778,7 +759,7 @@ class VersionedObjectDictCompat(object):
 
     def get(self, key, value=_NotSpecifiedSentinel):
         if key not in self.obj_fields:
-            raise AttributeError("'%s' object has no attribute '%s'" % (
+            raise AttributeError("'{}' object has no attribute '{}'".format(
                 self.__class__, key))
         if value != _NotSpecifiedSentinel and not self.obj_attr_is_set(key):
             return value
@@ -790,7 +771,7 @@ class VersionedObjectDictCompat(object):
             setattr(self, key, value)
 
 
-class ObjectListBase(collections.Sequence):
+class ObjectListBase(collections_abc.Sequence):
     """Mixin class for lists of objects.
 
     This mixin class can be added as a base class for an object that
@@ -808,7 +789,7 @@ class ObjectListBase(collections.Sequence):
     child_versions = {}
 
     def __init__(self, *args, **kwargs):
-        super(ObjectListBase, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if 'objects' not in kwargs:
             self.objects = []
             self._changed_fields.discard('objects')
@@ -978,8 +959,7 @@ class VersionedObjectSerializer(messaging.NoOpSerializer):
         return entity
 
 
-@six.add_metaclass(abc.ABCMeta)
-class VersionedObjectIndirectionAPI(object):
+class VersionedObjectIndirectionAPI(metaclass=abc.ABCMeta):
     def object_action(self, context, objinst, objmethod, args, kwargs):
         """Perform an action on a VersionedObject instance.
 
@@ -1167,7 +1147,11 @@ def obj_tree_get_versions(objname, tree=None):
         else:
             continue
 
-        obj_tree_get_versions(child_cls, tree=tree)
+        try:
+            obj_tree_get_versions(child_cls, tree=tree)
+        except IndexError:
+            raise exception.UnregisteredSubobject(
+                child_objname=child_cls, parent_objname=objname)
     return tree
 
 
